import { Group } from "@akarui/structures";
import { Command } from "../structures/Command.js";
import fs, { readFile } from "fs/promises";
import { Bundler } from "../index.js";
import Path from "path";
export class CommandManager {
    basic = new Group(Infinity);
    interaction = new Group(Infinity);
    ready = new Group(Infinity);
    debug = new Group(Infinity);
    component = new Group(Infinity);
    #client;
    constructor(client) {
        this.#client = client;
    }
    isValidType(type) {
        return ["basic", "interaction", "ready", "debug"].includes(type);
    }
    get types() {
        return ["basic", "interaction", "ready", "debug"];
    }
    static cmdTypes() {
        return ["basic", "interaction", "ready", "debug", "component"];
    }
    add(command) {
        if (!command.name)
            throw new Error("Command name is required");
        if (!command.type)
            throw new Error("Command type is required");
        if (!command.__path__)
            command.__path__ = "root";
        const cmd = new Command(command, this.#client);
        if (this.isValidType(command.type) && command.type !== "component")
            this[command.type].set(this[command.type].size, cmd);
        else {
            if (command.type === "component")
                this.component.set(command.name, cmd);
            else {
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                //@ts-ignore
                throw new Error("Invalid command type provided", {
                    ...command,
                });
            }
        }
    }
    addMany(commands) {
        for (const command of commands)
            this.add(command);
    }
    async load({ path, usingAoi = false, }) {
        const chalk = (await import("chalk")).default;
        const boxen = (await import("boxen")).default;
        const stats = await fs.stat(path);
        const Commands = [];
        if (!usingAoi) {
            if (stats.isDirectory()) {
                const files = await fs.readdir(path);
                for (const file of files) {
                    const filePath = path + "/" + file;
                    const stats = await fs.stat(filePath);
                    if (stats.isDirectory())
                        await this.load({ path: filePath, usingAoi });
                    else if (stats.isFile() &&
                        file.endsWith(".js") &&
                        !file.endsWith(".template.js")) {
                        // importing on windows
                        let command;
                        try {
                            command = await this.loadFile(filePath);
                            if (Array.isArray(command.default)) {
                                this.addMany(command.default);
                            }
                            else
                                this.add(command.default);
                            Commands.push({
                                path: filePath.split("/").pop(),
                                loaded: true,
                            });
                        }
                        catch (e) {
                            /* empty */
                            Commands.push({
                                path: filePath.split("/").pop(),
                                loaded: false,
                                reason: e,
                            });
                        }
                    }
                }
            }
        }
        else {
            if (stats.isDirectory()) {
                const files = await fs.readdir(path);
                for (const file of files) {
                    const filePath = path + "/" + file;
                    const stats = await fs.stat(filePath);
                    if (stats.isDirectory())
                        await this.load({ path: filePath, usingAoi });
                    else if (stats.isFile() &&
                        file.endsWith(".aoi") &&
                        !file.endsWith(".template.aoi")) {
                        const command = await readFile(filePath, "utf-8");
                        try {
                            const cmd = Bundler(command, this.#client).command;
                            cmd.__path__ = filePath;
                            this.add(cmd);
                            Commands.push({
                                path: filePath.split("/").pop(),
                                loaded: true,
                            });
                        }
                        catch (e) {
                            /* empty */
                            Commands.push({
                                path: filePath.split("/").pop(),
                                loaded: false,
                                reason: e,
                            });
                        }
                    }
                    // else if(stats.isFile() && file.endsWith(".aoi") && !file.endsWith(".template.aoi")) {
                    //     const command = await import(filePath);
                    //     this.add(command.default);
                    // }
                }
            }
        }
        const box = boxen(`${Commands.map((cmd) => {
            return `∷ ${chalk.cyanBright(cmd.loaded ? "Loaded" : "Failed")} ${chalk.greenBright(cmd.path)} ${chalk.redBright(cmd.loaded ? "" : cmd.reason)}`;
        }).join("\n")}
        `, {
            title: `∴ Loading ${chalk.blueBright(path)} ( ${chalk.yellowBright(Commands.length)} )`,
            borderStyle: "round",
            borderColor: "cyan",
            textAlignment: "left",
            backgroundColor: "black",
            width: 100,
            padding: 1,
            dimBorder: true,
            float: "center",
        });
        console.log(box);
    }
    async exec({ type, data, filter, }) {
        const cmd = this[type].filter((cmd) => filter(cmd));
        if (cmd.size) {
            for (const command of cmd.values()) {
                this.#client.managers.plugins.preCommand.forEach((plugin) => {
                    plugin.func({ command, data });
                });
                await command.__compiled__({ ...data, command });
                this.#client.managers.plugins.postCommand.forEach((plugin) => {
                    plugin.func({ command, data });
                });
            }
        }
    }
    async loadFile(filePath) {
        let command;
        try {
            command = require(filePath);
        }
        catch {
            if (process.platform === "win32") {
                const fp = Path.join("file:///", process.cwd(), filePath);
                command = await import(fp);
            }
            else {
                command = await import(Path.join(process.cwd(), filePath));
            }
        }
        return command;
    }
}
//# sourceMappingURL=Command.js.map